package com.eleadmin.common.core.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.write.handler.SheetWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteWorkbookHolder;
import com.eleadmin.common.core.exception.BusinessException;
import jakarta.servlet.http.HttpServletResponse;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.openxml4j.opc.PackagePartName;
import org.apache.poi.openxml4j.opc.PackageRelationship;
import org.apache.poi.openxml4j.opc.TargetMode;
import org.apache.poi.xssf.usermodel.XSSFPictureData;
import org.apache.poi.xssf.usermodel.XSSFRelation;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * excel 工具类
 *
 * @Author 1024创新实验室-主任:卓大
 * @Date 2024/4/22 22:49:07
 * @Wechat zhuoda1024
 * @Email lab1024@163.com
 * @Copyright 1024创新实验室 （ https://1024lab.net ），2012-2024
 */
public final class SmartExcelUtil {

    /**
     * 通用单sheet导出
     */
    public static void exportExcel(HttpServletResponse response, String fileName, String sheetName, Class head, Collection<?> data) throws IOException {
        // 设置下载消息头
        SmartResponseUtil.setDownloadFileHeader(response, fileName, null);
        // 下载
        EasyExcel.write(response.getOutputStream(), head)
                .autoCloseStream(Boolean.FALSE)
                .sheet(sheetName)
                .doWrite(data);
    }

    /**
     * 通用单 sheet水印 导出
     */
    public static void exportExcelWithWatermark(HttpServletResponse response, String fileName, String sheetName, Class head, Collection<?> data, String watermarkString) throws IOException {
        // 设置下载消息头
        SmartResponseUtil.setDownloadFileHeader(response, fileName, null);
        // 水印
        Watermark watermark = new Watermark(watermarkString);
        // 一定要inMemory
        EasyExcel.write(response.getOutputStream(), head)
                .inMemory(true)
                .sheet(sheetName)
                .registerWriteHandler(new CustomWaterMarkHandler(watermark))
                .doWrite(data);
    }


    @Slf4j
    private record CustomWaterMarkHandler(Watermark watermark) implements SheetWriteHandler {

        @Override
        public void afterSheetCreate(WriteWorkbookHolder writeWorkbookHolder, WriteSheetHolder writeSheetHolder) {
            BufferedImage bufferedImage = createWatermarkImage();
            XSSFWorkbook workbook = (XSSFWorkbook) writeSheetHolder.getParentWriteWorkbookHolder().getWorkbook();
            try {
                // 添加水印的具体操作
                addWatermarkToSheet(workbook, bufferedImage);
            } catch (Exception e) {
                log.error("添加水印出错:", e);
            }

        }

        /**
         * 创建水印图片
         *
         * @return BufferedImage
         */
        private BufferedImage createWatermarkImage() {
            // 获取水印相关参数
            Font font = watermark.getFont();
            int width = watermark.getWidth();
            int height = watermark.getHeight();
            Color color = watermark.getColor();
            String text = watermark.getContent();

            // 创建带有透明背景的 BufferedImage
            BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = image.createGraphics();

            // 设置画笔字体、平滑、颜色
            g.setFont(font);
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g.setColor(color);

            // 计算水印位置和角度
            int y = watermark.getYAxis();
            int x = watermark.getXAxis();
            AffineTransform transform = AffineTransform.getRotateInstance(Math.toRadians(-watermark.getAngle()), 0, y);
            g.setTransform(transform);
            // 绘制水印文字
            g.drawString(text, x, y);

            // 释放资源
            g.dispose();

            return image;
        }

        private void addWatermarkToSheet(XSSFWorkbook workbook, BufferedImage watermarkImage) {
            try (ByteArrayOutputStream os = new ByteArrayOutputStream()) {
                ImageIO.write(watermarkImage, "png", os);
                int pictureIdx = workbook.addPicture(os.toByteArray(), XSSFWorkbook.PICTURE_TYPE_PNG);
                XSSFPictureData pictureData = workbook.getAllPictures().get(pictureIdx);
                for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
                    // 获取每个Sheet表
                    XSSFSheet sheet = workbook.getSheetAt(i);
                    PackagePartName ppn = pictureData.getPackagePart().getPartName();
                    String relType = XSSFRelation.IMAGES.getRelation();
                    PackageRelationship pr = sheet.getPackagePart().addRelationship(ppn, TargetMode.INTERNAL, relType, null);
                    sheet.getCTWorksheet().addNewPicture().setId(pr.getId());
                }
            } catch (Exception e) {
                // 处理ImageIO.write可能抛出的异常
                log.error("添加水印图片时发生错误", e);
            }
        }
    }

    @Data
    private static class Watermark {

        public Watermark(String content) {
            this.content = content;
            init();
        }

        public Watermark(String content, Color color, Font font, double angle) {
            this.content = content;
            this.color = color;
            this.font = font;
            this.angle = angle;
            init();
        }

        /**
         * 根据水印内容长度自适应水印图片大小，简单的三角函数
         */
        private void init() {
            FontMetrics fontMetrics = new JLabel().getFontMetrics(this.font);
            int stringWidth = fontMetrics.stringWidth(this.content);
            int charWidth = fontMetrics.charWidth('A');
            this.width = (int) Math.abs(stringWidth * Math.cos(Math.toRadians(this.angle))) + 5 * charWidth;
            this.height = (int) Math.abs(stringWidth * Math.sin(Math.toRadians(this.angle))) + 5 * charWidth;
            this.yAxis = this.height;
            this.xAxis = charWidth;
        }

        /**
         * 水印内容
         */
        private String content;

        /**
         * 画笔颜色
         */
        private Color color = new Color(239, 239, 239);

        /**
         * 字体样式
         */
        private Font font = new Font("Microsoft YaHei", Font.BOLD, 26);

        /**
         * 水印宽度
         */
        private int width;

        /**
         * 水印高度
         */
        private int height;

        /**
         * 倾斜角度，非弧度制
         */
        private double angle = 25;

        /**
         * 字体的y轴位置
         */
        private int yAxis = 50;

        /**
         * 字体的X轴位置
         */
        private int xAxis;

        /**
         * 水平倾斜度
         */
        private double shearX = 0.1;

        /**
         * 垂直倾斜度
         */
        private double shearY = -0.26;
    }


    /**
     * 读取Excel文件并验证表头
     *
     * @param inputStream     Excel文件输入流
     * @param expectedHeaders 预期的表头
     * @param dataClass       Excel数据对应的类
     * @param <T>             数据实体的类型
     * @return 包含Excel数据的列表
     */
    public static <T> java.util.List<T> readExcelWithHeaderValidation(InputStream inputStream, java.util.List<String> expectedHeaders, Class<T> dataClass) {
        // 创建HeaderValidationListener实例
        HeaderValidationListener<T> headerValidationListener = new HeaderValidationListener<>(expectedHeaders);
        // 读取表头并验证 // 读取Excel数据
        return EasyExcel.read(inputStream, dataClass, headerValidationListener).sheet().doReadSync();
    }

    /**
     * 读取Excel文件并验证表头
     *
     * @param inputStream Excel文件输入流
     * @param dataClass   Excel数据对应的类
     * @param <T>         数据实体的类型
     * @return 包含Excel数据的列表
     * @throws IOException              如果读取文件时发生IO异常
     * @throws IllegalArgumentException 如果表头格式不正确
     */
    public static <T> java.util.List<T> readExcelWithHeaderValidation(InputStream inputStream, Class<T> dataClass) throws IOException {
        // 获取预期的表头
        java.util.List<String> expectedHeaders = getExpectedHeaders(dataClass);
        return readExcelWithHeaderValidation(inputStream, expectedHeaders, dataClass);
    }

    /**
     * 获取预期的表头
     *
     * @param dataClass Excel数据对应的类
     * @return 预期的表头列表
     */
    private static <T> java.util.List<String> getExpectedHeaders(Class<T> dataClass) {
        return java.util.Arrays.stream(dataClass.getDeclaredFields())
                .filter(field -> field.isAnnotationPresent(ExcelProperty.class))
                .map(field -> field.getAnnotation(ExcelProperty.class).value()[0])
                .collect(Collectors.toList());
    }

    /**
     * 自定义的ReadListener来读取表头并进行验证
     */
    private static class HeaderValidationListener<T> extends AnalysisEventListener<T> {

        private final java.util.List<String> expectedHeaders;
        private final java.util.List<String> actualHeaders = new java.util.ArrayList<>();
        private boolean headerRead = false;

        public HeaderValidationListener(java.util.List<String> expectedHeaders) {
            this.expectedHeaders = expectedHeaders;
        }

        @Override
        public void invokeHeadMap(java.util.Map<Integer, String> headMap, AnalysisContext context) {
            if (!headerRead) {
                actualHeaders.addAll(headMap.values());
                headerRead = true;
            }
        }

        @Override
        public void invoke(T t, AnalysisContext analysisContext) {

        }

        @Override
        public void doAfterAllAnalysed(AnalysisContext context) {
            // 验证表头
            for (int i = 0; i < expectedHeaders.size(); i++) {
                if (actualHeaders.size() <= i || !expectedHeaders.get(i).equals(actualHeaders.get(i))) {
                    throw new BusinessException("<p>你导入的excel表头与系统模板不一致，第<span style='color: red; fontWeight: bold;'>" + (i + 1) + "</span>列应是<span style='color: red; fontWeight: bold;'>" + expectedHeaders.get(i) + "</span></p>");
                }
            }
        }
    }
}
