package com.eleadmin.common.core.utils;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Component
public class SmartMybatisUtil {

    private static SqlSessionFactory sqlSessionFactory;

    public SmartMybatisUtil(SqlSessionFactory sqlSessionFactory) {
        SmartMybatisUtil.sqlSessionFactory = sqlSessionFactory;
    }

    /**
     * 批量插入数据
     *
     * @param mapperInterface MyBatis Mapper接口类
     * @param dataList        要插入的数据列表
     * @param batchSize       每批次插入的数据量
     * @param <T>             数据类型
     * @param <M>             Mapper接口类型
     */
    public static <T, M extends BaseMapper<T>> void batchInsert(Class<M> mapperInterface, List<T> dataList, int batchSize) {
        if (CollectionUtils.isEmpty(dataList)) {
            return;
        }
        try (SqlSession sqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH, false)) {
            M mapper = sqlSession.getMapper(mapperInterface);
            for (int i = 0; i < dataList.size(); i++) {
                mapper.insert(dataList.get(i));
                if (i % batchSize == 0) {
                    sqlSession.commit();
                    sqlSession.clearCache();
                }
            }
            sqlSession.commit();
        } catch (Exception e) {
            throw new RuntimeException("Batch insert failed", e);
        }
    }

    /**
     * 执行批量更新操作
     * 该方法用于高效地批量更新数据列表中的实体，通过指定的映射接口和批次大小
     * 主要目的是减少数据库的访问次数，提高更新效率
     *
     * @param mapperInterface 映射接口的Class类型，用于获取映射器
     * @param dataList 待更新的数据列表
     * @param batchSize 每次批量更新的数量
     * @param <T> 数据实体的类型
     * @param <M> 映射接口的类型，必须继承自BaseMapper
     */
    public static <T, M extends BaseMapper<T>> void batchUpdate(Class<M> mapperInterface, List<T> dataList, int batchSize) {
        // 检查数据列表是否为空，如果为空则直接返回
        if (CollectionUtils.isEmpty(dataList)) {
            return;
        }
        try {
            // 使用try-with-resources确保SqlSession在操作完成后会被正确关闭
            // 打开一个批处理类型的SqlSession，不自动提交事务
            try (SqlSession sqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH, false)) {
                // 获取映射接口对应的映射器实例
                M mapper = sqlSession.getMapper(mapperInterface);
                // 遍历数据列表，执行批量更新操作
                for (int i = 0; i < dataList.size(); i++) {
                    // 调用映射器的按ID更新方法
                    mapper.updateById(dataList.get(i));
                    // 每处理batchSize数量的数据后，提交一次事务并清空缓存
                    if (i % batchSize == 0) {
                        sqlSession.commit();
                        sqlSession.clearCache();
                    }
                }
                // 最后，确保所有更新操作都已提交
                sqlSession.commit();
            }
        } catch (Exception e) {
            // 如果发生异常，抛出运行时异常，包含异常信息
            throw new RuntimeException("Batch update failed", e);
        }
    }

}
