package com.eleadmin.common.core.utils;


import jakarta.servlet.http.HttpServletResponse;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author 张小张
 * @describe 压缩多个文件
 * @date 2023/07/18 18:10
 */
public class ZipUtils {
    public static void downloadAndZipFiles(HttpServletResponse response, List<String> filePaths, String zipFileName) {
        try {
            // 设置响应头，告诉浏览器这是一个压缩文件，用于下载
            response.setContentType("application/zip");
            response.setHeader("Content-Disposition", "attachment; filename=" + zipFileName);

            // 创建ZipOutputStream对象
            ZipOutputStream zipOut = new ZipOutputStream(response.getOutputStream());
            Map<String, Integer> existingFileNames = new HashMap<>();

            // 将文件逐个添加到压缩包中
            for (String filePath : filePaths) {
                File file = new File(filePath);
                if (file.exists()) {
                    addToZip(zipOut, file, existingFileNames);
                }
            }

            // 关闭ZipOutputStream
            zipOut.close();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void addToZip(ZipOutputStream zipOut, File file, Map<String, Integer> existingFileNames) throws IOException {
        if (file.isFile()) {
            FileInputStream fis = new FileInputStream(file);

            // 获取文件名，并对文件名进行处理
            String fileName = file.getName();
            fileName = removeContentBetweenUnderscoreAndDot(fileName);

            // 处理重名文件
            if (existingFileNames.containsKey(fileName)) {
                int count = existingFileNames.get(fileName);
                count++;
                fileName = addCountToFileName(fileName, count);
                existingFileNames.put(fileName, count);
            } else {
                existingFileNames.put(fileName, 1);
            }

            // 添加到压缩包
            zipOut.putNextEntry(new ZipEntry(fileName));

            byte[] bytes = new byte[1024];
            int length;
            while ((length = fis.read(bytes)) >= 0) {
                zipOut.write(bytes, 0, length);
            }
            fis.close();
            zipOut.closeEntry();
        }
    }

    private static String removeContentBetweenUnderscoreAndDot(String fileName) {
        // 查找第一个'_'的位置
        int underscoreIndex = fileName.indexOf('_');

        // 查找第一个'.'的位置
        int dotIndex = fileName.indexOf('.');

        // 如果找到了'_'和'.'，并且'_'在'.'之前，执行去除操作
        if (underscoreIndex != -1 && dotIndex != -1 && underscoreIndex < dotIndex) {
            // 获取'_'和'.'之间的内容
            String contentToRemove = fileName.substring(underscoreIndex, dotIndex);

            // 替换掉'_'和'.'之间的内容为空字符串
            fileName = fileName.replace(contentToRemove, "");
        }

        return fileName;
    }

    private static String addCountToFileName(String fileName, int count) {
        int dotIndex = fileName.lastIndexOf('.');
        if (dotIndex != -1) {
            String name = fileName.substring(0, dotIndex);
            String extension = fileName.substring(dotIndex);
            return name + "(" + count + ")" + extension;
        } else {
            return fileName + "(" + count + ")";
        }
    }

    /**
     * 文件夹压缩成ZIP
     *
     * @param zipFileName      生成压缩文件夹路径
     * @param sourceFileName   要压缩的文件路径
     * @param KeepDirStructure 是否保留原来的目录结构,true:保留目录结构;
     *                         false:所有文件跑到压缩包根目录下(注意：不保留目录结构可能会出现同名文件,会压缩失败)
     * @throws RuntimeException 压缩失败会抛出运行时异常
     */
    public static Boolean folderToZip(String zipFileName, String sourceFileName, boolean KeepDirStructure) {
        Boolean result = true;
        long start = System.currentTimeMillis();//开始
        ZipOutputStream zos = null;
        try {
            FileOutputStream fileOutputStream = new FileOutputStream(zipFileName);
            zos = new ZipOutputStream(fileOutputStream);
            File sourceFile = new File(sourceFileName);
            compress(sourceFile, zos, sourceFile.getName(), KeepDirStructure);
            long end = System.currentTimeMillis();//结束
            System.out.println("压缩完成，耗时：" + (end - start) + " 毫秒");
        } catch (Exception e) {
            result = false;
            e.printStackTrace();
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (IOException e) {
                    e.getStackTrace();
                }
            }
        }
        return result;
    }

    /**
     * 递归压缩方法
     *
     * @param sourceFile       源文件
     * @param zos              zip输出流
     * @param name             压缩后的名称
     * @param KeepDirStructure 是否保留原来的目录结构,true:保留目录结构;
     *                         false:所有文件跑到压缩包根目录下(注意：不保留目录结构可能会出现同名文件,会压缩失败)
     * @throws Exception
     */
    public static void compress(File sourceFile, ZipOutputStream zos, String name,
                                boolean KeepDirStructure) throws Exception {

        if (sourceFile.isFile()) {
            // 向zip输出流中添加一个zip实体，构造器中name为zip实体的文件的名字
            zos.putNextEntry(new ZipEntry(name));
            // copy文件到zip输出流中
            int len;
            byte[] buf = new byte[1024];
            FileInputStream in = new FileInputStream(sourceFile);
            while ((len = in.read(buf)) != -1) {
                zos.write(buf, 0, len);
            }
            // Complete the entry
            zos.closeEntry();
            in.close();
        } else {
            File[] listFiles = sourceFile.listFiles();
            if (listFiles == null || listFiles.length == 0) {
                // 需要保留原来的文件结构时,需要对空文件夹进行处理
                if (KeepDirStructure) {
                    // 空文件夹的处理
                    zos.putNextEntry(new ZipEntry(name + "/"));
                    // 没有文件，不需要文件的copy
                    zos.closeEntry();
                }
            } else {
                for (File file : listFiles) {
                    // 判断是否需要保留原来的文件结构
                    if (KeepDirStructure) {
                        // 注意：file.getName()前面需要带上父文件夹的名字加一斜杠,
                        // 不然最后压缩包中就不能保留原来的文件结构,即：所有文件都跑到压缩包根目录下了
                        compress(file, zos, name + "/" + file.getName(), KeepDirStructure);
                    } else {
                        compress(file, zos, file.getName(), KeepDirStructure);
                    }
                }
            }
        }
    }

    public static void copyFolder(File sourceFolder, File targetFolder) throws IOException {
        if (sourceFolder.isDirectory()) {
            if (!targetFolder.exists()) {
                targetFolder.mkdirs();
            }
            for (File file : sourceFolder.listFiles()) {
                if (file.isDirectory()) {
                    copyFolder(file, new File(targetFolder + File.separator + file.getName()));
                } else {
                    Files.copy(file.toPath(), Paths.get(targetFolder + File.separator + file.getName()), StandardCopyOption.REPLACE_EXISTING);
                }
            }
        }else {
            if (!targetFolder.exists()) {
                targetFolder.mkdirs();
            }
            Files.copy(sourceFolder.toPath(), Paths.get(targetFolder + File.separator + sourceFolder.getName()), StandardCopyOption.REPLACE_EXISTING);
        }
    }
}
