package com.eleadmin.commonflow;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.StreamProgress;
import com.alibaba.fastjson.JSON;
import com.eleadmin.api.MessageConfig;
import com.eleadmin.common.core.utils.PhoneNumberUtil;
import com.eleadmin.common.core.utils.Tuple2;
import com.eleadmin.common.core.web.*;
import com.eleadmin.commonflow.from.FlowableParam;
import com.eleadmin.distributiondeviceinfo.domain.entity.DistributionDeviceInfoEntity;
import com.eleadmin.distributiondeviceinfo.service.DistributionDeviceInfoService;
import com.eleadmin.flowable.dto.FlowAssignDTO;
import com.eleadmin.flowable.dto.FlowCompleteDTO;
import com.eleadmin.flowable.service.FlowableService;
import com.eleadmin.flowable.vo.ProcessProgressDetail;
import com.eleadmin.flowable.vo.TaskInfo;
import com.eleadmin.national_holidays.service.NationalHolidayService;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import com.eleadmin.outagework.service.OutageWorkService;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.powerdebug.service.PowerDebugService;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@RestController
@RequestMapping("/api/common-flow")
@RequiredArgsConstructor
@Slf4j
public class CommonFlowController extends BaseController {

    private final FlowableService flowableService;
    private final OutageWorkService outageWorkService;
    private final PowerDebugService powerDebugService;
    private final DistributionDeviceInfoService deviceInfoService;
    private final NationalHolidayService nationalHolidayService;
    private final MessageConfig.MessageService messageService;

    /**
     * 定时任务
     */
    @Scheduled(cron = "0 0 17 * * ?")
    public void sendMessage() {
        List<Task> tasks = flowableService.taskService.createTaskQuery().taskName("资料收集").list();
        if (tasks.isEmpty()) {
            return;
        }
        Set<String> list = tasks.stream().map(Task::getProcessInstanceId).collect(Collectors.toSet());

        DateTime dateTime = nationalHolidayService.subWorkday(new Date(), 2, true);

        List<OutageWorkEntity> entities = outageWorkService.lambdaQuery().in(OutageWorkEntity::getProcessInstanceId, list).ge(OutageWorkEntity::getWorkStartTime, dateTime).list();

        if (entities.isEmpty()) {
            return;
        }
        List<MessageConfig.Message> messages = entities.stream().flatMap(entity -> {
            String remarks = entity.getRemarks();
            List<String> numbers = PhoneNumberUtil.extractPhoneNumbers(remarks);
            if (numbers.isEmpty()) {
                return Stream.empty();
            }
            String format = entity.getWorkStartTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            String line = entity.getLineName();
            String content = "您有一条（" + format + " " + line + "）的工作未提供（信息表/台账）资料。请及时到图模维护工具处理。";

            return numbers.stream()
                    .map(number -> {
                        var message = new MessageConfig.Message();
                        message.setTitle("图模通知");
                        message.setContent(content);
                        message.setPhone(number);
                        return message;
                    });
        }).toList();
        messageService.push(MessageConfig.buildMessage(messages));

    }

    /**
     * 获取待办任务
     *
     * @param param 分页参数
     * @return 分页结果，包含待办任务列表
     */
    @RequestMapping("/getTodoTasks")
    public PageResult<TaskInfo> getTodoTasks(@RequestBody FlowableParam param) {
        // 获取当前用户的待办任务
        param.setUserId(getLoginUserId().toString());
        Tuple2<Long, List<Task>> tasks = flowableService.getTodoTasks(param);
        List<Task> taskList = tasks.t2;

        // 如果没有待办任务，返回空列表
        if (taskList.isEmpty()) {
            return new PageResult<>(Collections.emptyList(), tasks.t1);
        }

        // 构建任务信息列表
        return buildTaskInfoResult(taskList, tasks.t1);
    }

    /**
     * 获取已完成的任务
     *
     * @param param 分页参数
     * @return 分页结果，包含已完成任务列表
     */
    @RequestMapping("/getCompletedTasks")
    public PageResult<Map<String, Object>> getCompletedTasks(FlowableParam param) {
        // 获取当前用户的已完成任务
        param.setUserId(getLoginUserId().toString());

        PageResult<OutageWorkEntity> tasks = flowableService.getCompletedTasks(param);

        List<OutageWorkEntity> list = tasks.getList();

        Map<Integer, DistributionDeviceInfoEntity> collect;

        if (list.isEmpty()) {
            collect = null;
        } else {
            List<Integer> ids = list.stream().map(OutageWorkEntity::getId).toList();
            collect = deviceInfoService.lambdaQuery().in(DistributionDeviceInfoEntity::getOutageId, ids).list().stream().collect(Collectors.toMap(DistributionDeviceInfoEntity::getOutageId, it -> it));
        }

        List<Map<String, Object>> result = new ArrayList<>(list.size());

        for (OutageWorkEntity work : list) {
            DistributionDeviceInfoEntity deviceInfo = collect.get(work.getId());
            if (deviceInfo != null) {
                work.setDeviceInfo(deviceInfo);
            }
            result.add(Map.of("businessObject", work));
        }

        return new PageResult<>(result, tasks.getCount());
    }

    /**
     * 完成待办任务
     *
     * @param param param
     * @return
     */
    @PostMapping("/completeTask")
    public ApiResult<?> completeTask(@RequestBody @Valid FlowCompleteDTO param) {
        flowableService.completeTask(param);
        return success();
    }

    /**
     * 分配任务
     *
     * @param param param
     * @return
     */
    @PostMapping("/assignTask")
    public ApiResult<?> assignTask(@RequestBody @Valid FlowAssignDTO param) {
        flowableService.assignTask(param);
        return success();
    }

    /**
     * 获取流程详情
     *
     * @param id 流程ID
     * @return ProcessDetail
     */
    @GetMapping("/getProcessDetail/{id}")
    public ApiResult<?> getProcessDetail(@PathVariable String id) {
        return success(flowableService.getProcessProgressDetail(id));
    }

    /**
     * 获取流程图
     *
     * @param id 流程ID
     */
    @GetMapping("/getProcessDiagram/{id}")
    public void getProcessDetail(@PathVariable String id, HttpServletResponse response) throws IOException {
        try (InputStream diagram = flowableService.getProcessDiagram(id);
             OutputStream outputStream = response.getOutputStream()) {
            // 设置响应头
            response.setContentType("image/png"); // 根据实际图片类型设置
            response.setHeader("Content-Disposition", "inline; filename=process_diagram.png"); // 指定文件名

            if (diagram == null) {
                return;
            }

            IoUtil.copyByNIO(diagram, outputStream, 1024, new StreamProgress() {
                @Override
                public void start() {
                    // 可以添加启动时的逻辑
                }

                @Override
                public void progress(long l, long l1) {
                    // 可以添加进度更新时的逻辑
                }

                @Override
                public void finish() {
                    // 可以添加完成时的逻辑
                }
            });
        } catch (IOException e) {
            // 处理异常，例如记录日志或返回错误信息
            log.warn("获取流程图时发生错误", e);
        }
    }

    /**
     * 完成分配OMS编号和发起子流程
     */
    @PostMapping("/startSubProcess")
    @Transactional
    public ApiResult<?> startSubProcess(@RequestBody @Valid FlowCompleteDTO param) {
        ProcessProgressDetail detail = flowableService.getProcessDetailVyTaskId(param.getTaskId());
        if (detail == null) {
            return ApiResult.fail().setData("流程不存在");
        }
        String userId = getLoginUserId().toString();

        Object isAgree = param.getVariables().get("isAgree");
        if (isAgree == null) {
            return ApiResult.fail().setData("请选择是否同意");
        }

        Map<String, Object> variables = detail.getProcessVariables();

        @SuppressWarnings("unchecked")
        Map<String, String[]> startedSubProcesses = (Map<String, String[]>) variables.get("startedSubProcesses");
        if (startedSubProcesses == null) {
            startedSubProcesses = new HashMap<>();
        }

        if (isAgree.equals(false)) {
            if (!startedSubProcesses.isEmpty()) {
                return ApiResult.fail().setData("子流程已创建，无法退回");
            }
            flowableService.completeTask(param.getTaskId(), userId, param.getComment(), Map.of("isAgree", isAgree));
            return success();
        }

        String processInstanceId = detail.getProcessInstanceId();

        Map<String, Object> map = Map.of(
                "isTest", variables.get("isTest"),
                "mainProcessInstanceId", processInstanceId,
                "mainOperatingDepartment", variables.get("mainOperatingDepartment"),
                "tmhzz", variables.get("tmhzz")
        );
        // 启动子流程
        String subProcess = flowableService.startProcess("sub-flow", detail.getBusinessKey(), map);

        @SuppressWarnings("unchecked")
        List<String> omsNumber = (List<String>) param.getVariables().get("omsNumber");

        // 更新已启动的子流程记录
        startedSubProcesses.put(subProcess, omsNumber.toArray(new String[0]));


        // 移除已领取的子项
        @SuppressWarnings("unchecked")
        List<String> availableItems = new ArrayList<>((List<String>) detail.getProcessVariables().get("availableItems"));
        availableItems.removeIf(omsNumber::contains);
        flowableService.setVariables(processInstanceId, Map.of("startedSubProcesses", startedSubProcesses, "availableItems", availableItems));

        //完成当前任务
        flowableService.completeTask(param.getTaskId(), userId, param.getComment(), Map.of("isAgree", isAgree));

        //完成第一步的子流程
        FlowableParam flowableParam = new FlowableParam();
        flowableParam.setUserId(userId);
        flowableParam.setPage(1L);
        flowableParam.setLimit(1L);
        flowableParam.setProcessInstanceId(subProcess);

        Task task = flowableService.getTodoTasks(flowableParam).t2.get(0);
        flowableService.completeTask(task.getId(), userId, param.getComment(), param.getVariablesLocal(), param.getVariables());

        return success();
    }

    /**
     * 统计资料收集数量
     */
    @GetMapping("/countRecentCollectionTasks")
    public ApiResult<?> countRecentCollectionTasks() {
        List<String> list = flowableService.taskService.createTaskQuery().taskName("资料收集").taskCandidateOrAssigned(getLoginUserId().toString()).active().list().stream().map(org.flowable.task.api.TaskInfo::getProcessInstanceId).toList();
        Long count = outageWorkService.lambdaQuery().in(OutageWorkEntity::getProcessInstanceId, list).apply("now() > work_start_time - INTERVAL 3 DAY").count();
        /*
        * .apply("now() > work_start_time - INTERVAL (CASE " +
                        "WHEN WEEKDAY(work_start_time) = 5 THEN 4 " +   // 周六 -> 减4天到周三
                        "WHEN WEEKDAY(work_start_time) = 6 THEN 5 " +   // 周日 -> 减5天到周三
                        "ELSE 3 " +                                     // 正常工作日 -> 减3天
                        "END) DAY")
        * */
        return success(count);
    }

    /**
     * 构建任务信息列表
     *
     * @param taskList 任务列表
     * @param total    总任务数
     * @return 分页结果，包含任务信息列表
     */
    private PageResult<TaskInfo> buildTaskInfoResult(List<? extends Task> taskList, Long total) {
        // 获取所有任务的流程实例ID
        Set<String> processIds = taskList.stream().map(Task::getProcessInstanceId).collect(Collectors.toSet());

        // 查询流程实例的业务键
        Map<String, String> businessKeys = flowableService.queryProcesses(processIds).stream().collect(Collectors.toMap(ProcessInstance::getProcessInstanceId, ProcessInstance::getBusinessKey));

        if (businessKeys.isEmpty()) {
            throw new RuntimeException("没有找到对应流程实例的业务键");
        }

        // 根据业务键查询对应的OutageWorkEntity
        Map<String, OutageWorkEntity> outageWorkMap = outageWorkService.listRel(businessKeys.values()).stream().collect(Collectors.toMap(it -> it.getId().toString(), v -> v));

        // 获取所有OutageWorkEntity的ID
        List<Integer> outageIds = outageWorkMap.values().stream().map(OutageWorkEntity::getId).toList();

        if (outageIds.isEmpty()) {
            log.error("=================================================");
            log.error(JSON.toJSONString(businessKeys));
            throw new RuntimeException("没有找到对应流程实例的业务键对应的OutageWorkEntity");
        }

        Map<Integer, DistributionDeviceInfoEntity> deviceInfoMap = deviceInfoService.lambdaQuery()
                .in(DistributionDeviceInfoEntity::getOutageId, outageIds)
                .list()
                .stream()
                .collect(Collectors.toMap(DistributionDeviceInfoEntity::getOutageId, it -> it));

        // 构建TaskInfo列表并返回
        return new PageResult<>(taskList.stream().map(task -> {
            TaskInfo taskInfo = TaskInfo.of(task);
            OutageWorkEntity outageWork = outageWorkMap.get(businessKeys.get(task.getProcessInstanceId()));
            if (outageWork == null) {
                return taskInfo;
            }
            DistributionDeviceInfoEntity deviceInfo = deviceInfoMap.get(outageWork.getId());
            if (deviceInfo != null) {
                Object availableItems = flowableService.getVariable(task.getProcessInstanceId(), "availableItems");
                if (availableItems != null) {
                    @SuppressWarnings("unchecked")
                    List<String> items = (List<String>) availableItems;
                    deviceInfo.setOmsNumber(items);
                }
                outageWork.setDeviceInfo(deviceInfo);
            }

            taskInfo.setBusinessObject(outageWork);
            return taskInfo;
        }).toList(), total);
    }

}
