package com.eleadmin.flowable.service;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eleadmin.common.core.utils.Tuple2;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.common.system.entity.User;
import com.eleadmin.common.system.service.UserService;
import com.eleadmin.commonflow.from.FlowableParam;
import com.eleadmin.flowable.dao.FlowableDao;
import com.eleadmin.flowable.dto.FlowAssignDTO;
import com.eleadmin.flowable.dto.FlowCompleteDTO;
import com.eleadmin.flowable.vo.ProcessProgressDetail;
import com.eleadmin.flowable.vo.TaskHistory;
import com.eleadmin.flowable.vo.TaskInfo;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.engine.*;
import org.flowable.engine.form.FormProperty;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.history.HistoricProcessInstanceQuery;
import org.flowable.engine.repository.Deployment;
import org.flowable.engine.repository.DeploymentBuilder;
import org.flowable.engine.repository.DeploymentQuery;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.engine.runtime.ProcessInstanceQuery;
import org.flowable.engine.task.Comment;
import org.flowable.image.impl.DefaultProcessDiagramGenerator;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskInfoQuery;
import org.flowable.task.api.TaskQuery;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.flowable.variable.api.history.HistoricVariableInstance;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.InputStream;
import java.text.MessageFormat;
import java.util.*;
import java.util.stream.Collectors;

@RequiredArgsConstructor
@Service
@Slf4j
public class FlowableService {

    private final RuntimeService runtimeService;
    public final TaskService taskService;
    private final HistoryService historyService;
    private final RepositoryService repositoryService;
    private final FormService formService;
    private final UserService userService;
    private final FlowableDao flowableDao;

    /**
     * 完成任务
     *
     * @param taskId    任务ID
     * @param userId    用户ID
     * @param comment   审批意见
     * @param variables 流程变量
     */
    public void completeTask(String taskId, String userId, String comment, Map<String, Object> variables) {
        completeTask(taskId, userId, comment, null, variables);
    }

    /**
     * 完成任务
     *
     * @param taskId         任务ID
     * @param userId         用户ID
     * @param variablesLocal 任务变量
     * @param comment        审批意见
     * @param variables      流程变量
     */
    public void completeTask(String taskId, String userId, String comment, Map<String, Object> variablesLocal, Map<String, Object> variables) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();

        if (task == null) {
            log.error("任务不存在或已完成");
            return;
        }

        // 如果任务未分配给任何人，尝试领取任务
        if (task.getAssignee() == null) {
            taskService.claim(taskId, userId);
        }

        // 添加审批意见
        if (StringUtils.isNotBlank(comment)) {
            taskService.addComment(taskId, task.getProcessInstanceId(), comment);
        }

        // 设置流程变量
        if (variables != null) {
            runtimeService.setVariables(task.getProcessInstanceId(), variables);
        }

        // 设置任务变量
        if (variablesLocal != null) {
            taskService.setVariablesLocal(taskId, variablesLocal);
        }

        // 完成任务
        taskService.complete(taskId, userId);
    }

    /**
     * 查询待办任务（分页）
     */
    public Tuple2<Long, List<Task>> getTodoTasks(FlowableParam param) {
        TaskQuery query = taskService.createTaskQuery()
                .includeProcessVariables()
                .active()
                .orderByTaskCreateTime()
                .asc();

        String userId = param.getUserId();
        if (StringUtils.isNotBlank(userId)) {
            query.taskCandidateOrAssigned(userId).taskCandidateUser(userId);
        }

        applyCommonConditions(query, param);

        int page = param.getPage().intValue();
        int limit = param.getLimit().intValue();
        return Tuple2.of(query.count(), query.listPage((page - 1) * limit, limit));
    }

    /**
     * 查询已办任务（分页）
     */
    public PageResult<OutageWorkEntity> getCompletedTasks(FlowableParam param) {
        PageParam<Map<String, String>, FlowableParam> pageParam = new PageParam<>(param);
        List<OutageWorkEntity> list = flowableDao.getCompletedTaskBusinessKeys(pageParam, param);
        return new PageResult<>(list,  pageParam.getTotal());
    }

    private <T extends TaskInfoQuery<?, ?>> void applyCommonConditions(T query, FlowableParam param) {
        if (StringUtils.isNotBlank(param.getProcessInstanceId())) {
            query.processInstanceId(param.getProcessInstanceId());
        }
        if (StringUtils.isNotBlank(param.getTaskName())) {
            query.taskName(param.getTaskName());
        }
        if (StringUtils.isNotBlank(param.getLineName())) {
            query.processVariableValueLike("lineName", MessageFormat.format("%{0}%", param.getLineName()));
        }
        if (StringUtils.isNotBlank(param.getPowerOutageScope())) {
            query.processVariableValueLike("powerOutageScope", MessageFormat.format("%{0}%", param.getPowerOutageScope()));
        }
        if (StringUtils.isNotBlank(param.getInspectionContent())) {
            query.processVariableValueLike("inspectionContent", MessageFormat.format("%{0}%", param.getInspectionContent()));
        }
        if (param.getEndDateBegin() != null) {
            query.processVariableValueGreaterThanOrEqual("endDateBegin", param.getEndDateBegin());
        }
        if (param.getEndDateEnd() != null) {
            query.processVariableValueLessThanOrEqual("endDateEnd", param.getEndDateEnd());
        }
    }

    /**
     * 启动流程实例（增）
     *
     * @param processKey  流程定义KEY
     * @param businessKey 业务主键
     * @param variables   流程变量
     * @return 流程实例ID
     */
    public String startProcess(String processKey, String businessKey, Map<String, Object> variables) {
        ProcessInstance instance = runtimeService.startProcessInstanceByKey(processKey, businessKey, variables);
        return instance.getId();
    }

    /**
     * 删除流程实例和历史（删）
     *
     * @param processInstanceId 流程实例ID
     * @param deleteReason      删除原因
     */
    @Transactional
    public void deleteProcess(String processInstanceId, String deleteReason) {
        deleteProcess(Collections.singletonList(processInstanceId), deleteReason);
    }

    /**
     * 删除流程实例和历史（删）
     *
     * @param processInstanceIds 流程实例ID
     * @param deleteReason       删除原因
     */
    @Transactional
    public void deleteProcess(Collection<String> processInstanceIds, String deleteReason) {
        if (processInstanceIds == null || processInstanceIds.isEmpty()) {
            return;
        }
        HashSet<String> set = new HashSet<>(processInstanceIds);

        // 批量获取运行中的流程实例
        List<ProcessInstance> activeInstances = runtimeService.createProcessInstanceQuery().processInstanceIds(set).list();

        // 批量获取历史流程实例
        List<HistoricProcessInstance> historicInstances = historyService.createHistoricProcessInstanceQuery().processInstanceIds(set).list();

        Set<String> allInstanceIds = new HashSet<>();
        activeInstances.forEach(pi -> allInstanceIds.add(pi.getId()));
        historicInstances.forEach(hpi -> allInstanceIds.add(hpi.getId()));

        if (allInstanceIds.isEmpty()) {
            return;
        }

        // 删除运行中的流程实例
        if (!activeInstances.isEmpty()) {
            runtimeService.bulkDeleteProcessInstances(activeInstances.stream().map(ProcessInstance::getId).toList(), deleteReason);
        }

        // 删除历史流程实例
        if (!historicInstances.isEmpty()) {
            historyService.bulkDeleteHistoricProcessInstances(historicInstances.stream().map(HistoricProcessInstance::getId).toList());
        }
    }


    /**
     * 删除流程实例（删）
     *
     * @param processInstanceId 流程实例ID
     * @param deleteReason      删除原因
     */
    public void deleteProcessInstance(String processInstanceId, String deleteReason) {
        try {
            runtimeService.deleteProcessInstance(processInstanceId, deleteReason);
        } catch (Exception ignored) {
        }
    }

    /**
     * 更新流程变量（改）
     *
     * @param processInstanceId 流程实例ID
     * @param variables         新流程变量
     */
    public void updateVariables(String processInstanceId, Map<String, Object> variables) {
        runtimeService.setVariables(processInstanceId, variables);
    }

    /**
     * 查询流程实例（查）
     *
     * @param processKey  流程定义KEY
     * @param businessKey 业务主键
     */
    public List<ProcessInstance> queryProcesses(String processKey, String businessKey) {

        ProcessInstanceQuery query = runtimeService.createProcessInstanceQuery();

        if (processKey != null) {
            query.processDefinitionKey(processKey);
        }

        if (businessKey != null) {
            query.processInstanceBusinessKey(businessKey);
        }
        return query.orderByProcessInstanceId().desc().list();
    }

    /**
     * 查询流程实例（查）
     *
     * @param processKey 流程定义KEY
     */
    public List<ProcessInstance> queryProcesses(Set<String> processKey) {

        ProcessInstanceQuery query = runtimeService.createProcessInstanceQuery();

        if (processKey != null) {
            query.processInstanceIds(processKey);
        }
        return query.orderByStartTime().desc().list();
    }

    public List<HistoricProcessInstance> queryHistoryProcesses(Set<String> processKey) {
        HistoricProcessInstanceQuery query = historyService.createHistoricProcessInstanceQuery().notDeleted();

        if (processKey != null) {
            query.processInstanceIds(processKey);
        }
        return query.orderByProcessInstanceStartTime().desc().list();
    }

    /**
     * 获取单个流程详情
     */
    public ProcessInstance getProcessDetail(String processInstanceId) {
        return runtimeService.createProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();
    }

    /**
     * 获取多个流程详情
     */
    public List<ProcessInstance> getProcessDetails(Set<String> processInstanceId) {
        return runtimeService.createProcessInstanceQuery().processInstanceIds(processInstanceId).list();
    }

    /**
     * 生成流程图（带高亮）
     *
     * @param processInstanceId 流程实例ID
     * @return 流程图输入流
     */
    public InputStream getProcessDiagram(String processInstanceId) {
        final var font = "宋体";
        final var imageType = "png";
        final var scaleFactor = 1.0;
        ProcessInstance instance = runtimeService.createProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();
        if (instance != null) {
            // 获取当前活动的节点
            List<String> activeActivityIds = runtimeService.getActiveActivityIds(processInstanceId);

            /*// 获取已执行的节点
            List<HistoricTaskInstance> historicTasks = historyService.createHistoricTaskInstanceQuery()
                    .processInstanceId(processInstanceId)
                    .orderByHistoricTaskInstanceStartTime().asc()
                    .list();
            List<String> completedActivityIds = historicTasks.stream()
                    .map(HistoricTaskInstance::getTaskDefinitionKey)
                    .toList();

            // 获取已执行的连线
            List<String> highlightedFlows = historyService.createHistoricActivityInstanceQuery()
                    .processInstanceId(processInstanceId)
                    .activityType("sequenceFlow") // 过滤出连线类型的活动
                    .list()
                    .stream()
                    .map(HistoricActivityInstance::getActivityId) // 提取连线ID
                    .toList();

            // 合并当前活动和已执行的节点
            List<String> highlightedActivities = new ArrayList<>();
            highlightedActivities.addAll(activeActivityIds);
            highlightedActivities.addAll(completedActivityIds);*/

            // 获取BPMN模型
            BpmnModel bpmnModel = repositoryService.getBpmnModel(instance.getProcessDefinitionId());
            return new DefaultProcessDiagramGenerator().generateDiagram(bpmnModel, imageType, activeActivityIds, Collections.emptyList(), font, font, font, null, scaleFactor, true);
        }


        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();
        if (historicProcessInstance != null) {
            BpmnModel bpmnModel = repositoryService.getBpmnModel(historicProcessInstance.getProcessDefinitionId());
            // 生成流程图配置
            return new DefaultProcessDiagramGenerator().generateDiagram(bpmnModel, imageType, font, font, font, null, scaleFactor, true);
        }

        return null;

    }

    /**
     * 查询任务的历史审批意见
     *
     * @param taskId 任务ID
     * @return 审批意见列表
     */
    public List<Comment> getTaskComments(String taskId) {
        return taskService.getTaskComments(taskId);
    }

    /**
     * 查询流程进度详情
     *
     * @param processInstanceId 流程实例ID
     * @return 包含进度信息、当前节点、审批历史的详情对象
     */
    public ProcessProgressDetail getProcessProgressDetail(String processInstanceId) {
        ProcessInstance instance = runtimeService.createProcessInstanceQuery()
                .processInstanceId(processInstanceId)
                .includeProcessVariables()
                .singleResult();

        ProcessProgressDetail detail = new ProcessProgressDetail();

        if (instance == null) {
            // 如果运行中不存在，查询历史流程
            HistoricProcessInstance historicInstance = historyService.createHistoricProcessInstanceQuery()
                    .processInstanceId(processInstanceId)
                    .includeProcessVariables()
                    .singleResult();
            if (historicInstance == null) return null;

            // 构建已完成流程的进度详情

            detail.setProcessInstanceId(processInstanceId);
            detail.setBusinessKey(historicInstance.getBusinessKey());
            detail.setStartTime(historicInstance.getStartTime());
            detail.setStartUserId(historicInstance.getStartUserId());
            detail.setProcessVariables(historicInstance.getProcessVariables());
        } else {
            // 1. 流程基本信息
            detail.setProcessInstanceId(processInstanceId);
            detail.setBusinessKey(instance.getBusinessKey());
            detail.setStartTime(instance.getStartTime());
            detail.setStartUserId(instance.getStartUserId());
            detail.setProcessVariables(instance.getProcessVariables());
        }

        // 2. 当前任务信息
        List<Task> currentTasks = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
        detail.setCurrentTasks(currentTasks.stream().map(TaskInfo::of).toList());

        // 3. 历史任务信息
        List<HistoricTaskInstance> historicTasks = historyService.createHistoricTaskInstanceQuery()
                .processInstanceId(processInstanceId)
                .orderByHistoricTaskInstanceStartTime().asc()
                .list();

        List<TaskHistory> list = historicTasks.stream().map(TaskHistory::of).toList();
        Map<String, Map<String, Object>> variables = getHistoricTaskVariablesByProcessInstanceId(processInstanceId);

        Set<String> assignees = list.stream().map(TaskHistory::getAssignee).filter(Objects::nonNull).collect(Collectors.toSet());
        Map<Integer, String> userMap = userService.list(Wrappers.lambdaQuery(User.class).in(User::getUserId, assignees).select(User::getUserId, User::getNickname)).stream().collect(Collectors.toMap(User::getUserId, User::getNickname));

        for (TaskHistory history : list) {
            if (history.getAssignee() != null) {
                history.setNickname(userMap.getOrDefault(Integer.parseInt(history.getAssignee()), ""));
            }
            String comments = getTaskComments(history.getTaskId()).stream().map(Comment::getFullMessage).collect(Collectors.joining());
            history.setComment(comments);
            history.setBusinessObject(variables.get(history.getTaskId()));
        }

        detail.setHistoryTasks(list);

        return detail;
    }


    public Map<String, List<TaskInfo>> getCurrentTasks(Set<String> processInstanceId) {
        if (processInstanceId.isEmpty()) {
            return Collections.emptyMap();
        }
        List<Task> currentTasks = taskService.createTaskQuery().processInstanceIdIn(processInstanceId.stream().filter(Objects::nonNull).collect(Collectors.toSet())).list();
        return currentTasks.stream().collect(Collectors.groupingBy(Task::getProcessInstanceId)).entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().stream().map(TaskInfo::of).toList()));
    }

    public List<FormProperty> getTaskForm(String taskId) {
        return formService.getTaskFormData(taskId).getFormProperties();
    }

    /**
     * 查询流程部署定义
     *
     * @param deploymentName 部署名称（可选）
     * @param deploymentKey  部署Key（可选）
     * @return 部署定义列表
     */
    public List<Deployment> queryDeployments(String deploymentName, String deploymentKey) {
        DeploymentQuery query = repositoryService.createDeploymentQuery();

        if (StringUtils.isNotBlank(deploymentName)) {
            query.deploymentName(deploymentName);
        }

        if (StringUtils.isNotBlank(deploymentKey)) {
            query.deploymentKey(deploymentKey);
        }

        return query.orderByDeploymentTime().desc().list();
    }

    /**
     * 删除流程部署
     *
     * @param deploymentId 部署ID
     * @param cascade      是否级联删除（true删除关联的流程实例和历史数据）
     */
    public void deleteDeployment(String deploymentId, boolean cascade) {
        repositoryService.deleteDeployment(deploymentId, cascade);
    }

    /**
     * 部署流程定义
     *
     * @param deploymentName 部署名称（必填）
     * @param tenantId       租户ID（可选）
     * @param resources      资源文件（key:资源名称，value:资源输入流）
     * @return 部署对象
     */
    public Deployment deployProcess(String deploymentName, String tenantId, Map<String, InputStream> resources) {
        if (StringUtils.isBlank(deploymentName)) {
            throw new IllegalArgumentException("Deployment name cannot be empty");
        }
        if (resources == null || resources.isEmpty()) {
            throw new IllegalArgumentException("Process resources cannot be empty");
        }

        DeploymentBuilder deploymentBuilder = repositoryService.createDeployment()
                .name(deploymentName)
                .enableDuplicateFiltering(); // 开启重复部署过滤

        if (StringUtils.isNotBlank(tenantId)) {
            deploymentBuilder.tenantId(tenantId);
        }

        // 添加所有资源文件
        resources.forEach((resourceName, inputStream) -> {
            if (inputStream != null) {
                deploymentBuilder.addInputStream(resourceName, inputStream);
            }
        });

        return deploymentBuilder.deploy();
    }

    /**
     * 获取任务本地变量
     *
     * @param taskId 任务ID
     * @return 反序列化后的变量Map
     */
    public Map<String, Object> getTaskLocalVariables(String taskId) {
        return taskService.getVariablesLocal(taskId).entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> JSONUtil.parse(e.getValue())));
    }

    /**
     * 获取历史任务变量
     *
     * @param taskId 历史任务ID
     * @return 变量Map
     */
    public Map<String, Object> getHistoricTaskVariables(String taskId) {
        return historyService.createHistoricVariableInstanceQuery()
                .taskId(taskId)
                .list()
                .stream()
                .collect(Collectors.toMap(HistoricVariableInstance::getVariableName, v -> JSONUtil.parse(v.getValue())));
    }

    /**
     * 获取历史任务变量
     *
     * @param processInstanceId 历史任务ID
     * @return 变量Map
     */
    public Map<String, Map<String, Object>> getHistoricTaskVariablesByProcessInstanceId(String processInstanceId) {
        return historyService.createHistoricVariableInstanceQuery()
                .processInstanceId(processInstanceId)
                .list()
                .stream()
                // 过滤出有任务ID的变量（排除流程级变量）
                .filter(v -> v.getTaskId() != null)
                // 按任务ID分组
                .collect(Collectors.groupingBy(HistoricVariableInstance::getTaskId, Collectors.toMap(HistoricVariableInstance::getVariableName, HistoricVariableInstance::getValue)));
    }


    /**
     * 暂停流程实例
     *
     * @param processInstanceId 流程实例ID
     */
    public void suspendProcessInstance(String processInstanceId) {
        try {
            runtimeService.suspendProcessInstanceById(processInstanceId);
        } catch (Exception e) {
            log.warn("流程实例ID:{} 暂停失败", processInstanceId);
        }
    }

    /**
     * 继续流程实例
     *
     * @param processInstanceId 流程实例ID
     */
    public void resumeProcessInstance(String processInstanceId) {
        try {
            runtimeService.activateProcessInstanceById(processInstanceId);
        } catch (Exception e) {
            log.warn("流程实例ID:{} 继续失败", processInstanceId);
        }
    }


    public Long getOverdueWork(String taskName) {
        return taskService.createTaskQuery().taskName(taskName).active().count();
    }

    public ProcessProgressDetail getProcessDetailVyTaskId(@NotBlank String taskId) {
        Task task = taskService.createTaskQuery().taskId(taskId).active().singleResult();
        if (task == null) {
            return null;
        }
        return getProcessProgressDetail(task.getProcessInstanceId());
    }

    public void setVariable(String processInstanceId, String name, Object value) {
        runtimeService.setVariable(processInstanceId, name, value);
    }

    public void setVariables(String processInstanceId, Map<String, Object> variables) {
        runtimeService.setVariables(processInstanceId, variables);
    }

    public Object getVariable(String processInstanceId, String name) {
        try {
            return runtimeService.getVariable(processInstanceId, name);
        } catch (Exception e) {
            return null;
        }
    }

    public Object getHistoricVariables(String processInstanceId, String startedSubProcesses) {
        HistoricVariableInstance result = historyService.createHistoricVariableInstanceQuery().processInstanceId(processInstanceId).variableName(startedSubProcesses).singleResult();
        if (result == null) {
            return null;
        }
        return result.getValue();
    }

    public void completeTask(@Valid FlowCompleteDTO param) {
        String instanceId = taskService.createTaskQuery().taskId(param.getTaskId()).singleResult().getProcessInstanceId();
        completeTask(param.getTaskId(), param.getUserId(), param.getComment(), param.getVariablesLocal(), param.getVariables());
        Task nextTask = null;

        Boolean completeNextTask = param.getCompleteNextTask();

        String candidate = param.getCandidate();

        if (StringUtils.isNotBlank(candidate)) {
            nextTask = taskService.createTaskQuery().processInstanceId(instanceId).active().singleResult();
            if (nextTask != null) {
                taskService.setAssignee(nextTask.getId(), candidate);
            }
        }

        if (StringUtils.isNotBlank(param.getTmhzz())) {
            runtimeService.setVariable(instanceId, "tmhzz", param.getTmhzz());
        }

        if (completeNextTask != null && completeNextTask) {
            if (nextTask == null) {
                nextTask = taskService.createTaskQuery().processInstanceId(instanceId).active().singleResult();
            }
            if (nextTask != null) {
                completeTask(nextTask.getId(), param.getUserId(), param.getComment(), param.getVariablesLocal(), param.getVariables());
            }
        }

    }

    /**
     * 分配任务
     *
     * @param param 参数
     */
    public void assignTask(@Valid FlowAssignDTO param) {
        taskService.setAssignee(param.getTaskId(), param.getCandidate());
        String instanceId = taskService.createTaskQuery().taskId(param.getTaskId()).singleResult().getProcessInstanceId();
        runtimeService.setVariables(instanceId, param.getVariables());
    }

    public void setAssignee(String taskId, String candidateId) {
        taskService.setAssignee(taskId, candidateId);
    }
}

