package com.eleadmin.powerdebug.service;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eleadmin.common.core.exception.BusinessException;
import com.eleadmin.common.core.utils.SmartBeanUtil;
import com.eleadmin.common.core.utils.SmartExcelUtil;
import com.eleadmin.common.core.utils.SmartMybatisUtil;
import com.eleadmin.common.core.web.ApiResult;
import com.eleadmin.common.core.web.PageParam;
import com.eleadmin.common.core.web.PageResult;
import com.eleadmin.outagework.dao.OutageWorkDao;
import com.eleadmin.outagework.domain.entity.OutageWorkEntity;
import com.eleadmin.powerdebug.dao.PowerDebugDao;
import com.eleadmin.powerdebug.domain.entity.PowerDebugEntity;
import com.eleadmin.powerdebug.domain.form.PowerDebugAddForm;
import com.eleadmin.powerdebug.domain.form.PowerDebugQueryForm;
import com.eleadmin.powerdebug.domain.form.PowerDebugUpdateForm;
import com.eleadmin.powerdebug.domain.vo.PowerDebugExcelVO;
import com.eleadmin.powerdebug.domain.vo.PowerDebugSimpleVO;
import com.eleadmin.powerdebug.domain.vo.PowerDebugVO;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 电力设备调试工作记录表 Service
 *
 * @Author 王益亭
 * @Date 2025-03-03 10:05:18
 * @Copyright gxrd
 */

@Service
@Slf4j
public class PowerDebugService extends ServiceImpl<PowerDebugDao, PowerDebugEntity> implements IService<PowerDebugEntity> {

    @Resource
    private PowerDebugDao powerDebugDao;
    @Resource
    private OutageWorkDao outageWorkDao;

    /**
     * 分页查询
     *
     * @param queryForm
     * @return
     */
    public PageResult<PowerDebugVO> queryPage(PowerDebugQueryForm queryForm) {
        PageParam<PowerDebugEntity, PowerDebugQueryForm> tuPageParam = new PageParam<>(queryForm);
        List<PowerDebugVO> list = powerDebugDao.queryPage(tuPageParam, queryForm);
        return new PageResult<>(list, tuPageParam.getTotal());
    }

    public PageResult<PowerDebugSimpleVO> querySimplePage(PowerDebugQueryForm queryForm) {
        PageParam<PowerDebugEntity, PowerDebugQueryForm> tuPageParam = new PageParam<>(queryForm);
        List<PowerDebugSimpleVO> list = powerDebugDao.querySimplePage(tuPageParam, queryForm);
        return new PageResult<>(list, tuPageParam.getTotal());
    }

    /**
     * 添加
     */
    public ApiResult<String> add(PowerDebugAddForm addForm) {
        PowerDebugEntity powerDebugEntity = SmartBeanUtil.copy(addForm, PowerDebugEntity.class);
        setOutage(powerDebugEntity);
        powerDebugDao.insert(powerDebugEntity);
        return ApiResult.ok();
    }

    /**
     * 更新
     *
     * @param updateForm
     * @return
     */
    public ApiResult<String> update(PowerDebugUpdateForm updateForm) {
        PowerDebugEntity powerDebugEntity = SmartBeanUtil.copy(updateForm, PowerDebugEntity.class);
        setOutage(powerDebugEntity);
        powerDebugDao.updateById(powerDebugEntity);
        return ApiResult.ok();
    }

    /**
     * 批量删除
     *
     * @param idList
     * @return
     */
    @Transactional
    public ApiResult<String> batchDelete(List<Integer> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return ApiResult.ok();
        }
        List<PowerDebugEntity> entityList = lambdaQuery().select(PowerDebugEntity::getOutageId).in(PowerDebugEntity::getId, idList).list();
        List<Integer> integers = entityList.stream().filter(Objects::nonNull).map(PowerDebugEntity::getOutageId).filter(Objects::nonNull).toList();
        if (!integers.isEmpty()) {
            outageWorkDao.update(Wrappers.lambdaUpdate(OutageWorkEntity.class).set(OutageWorkEntity::getWhetherDebug, 0).in(OutageWorkEntity::getId,integers));
        }
        powerDebugDao.deleteByIds(idList);
        return ApiResult.ok();
    }

    /**
     * 单个删除
     */
    public ApiResult<String> delete(Integer id) {
        if (null == id) {
            return ApiResult.ok();
        }
        PowerDebugEntity entity = lambdaQuery().select(PowerDebugEntity::getOutageId).in(PowerDebugEntity::getId, id).one();
        if (entity != null && entity.getOutageId() != null) {
            outageWorkDao.update(Wrappers.lambdaUpdate(OutageWorkEntity.class).set(OutageWorkEntity::getWhetherDebug, 0).eq(OutageWorkEntity::getId,entity.getOutageId()));
        }
        powerDebugDao.deleteById(id);
        return ApiResult.ok();
    }

    public ApiResult<String> importFromExcel(MultipartFile file) {
        List<PowerDebugExcelVO> dataList;
        try {
            dataList = SmartExcelUtil.readExcelWithHeaderValidation(file.getInputStream(), PowerDebugExcelVO.class);
            // 处理导入的数据，例如保存到数据库
            List<PowerDebugEntity> list = SmartBeanUtil.copyList(dataList, PowerDebugEntity.class);

            for (PowerDebugEntity debug : list) {
                setOutage(debug);
            }

            SmartMybatisUtil.batchInsert(PowerDebugDao.class, list, 1000);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new BusinessException("数据格式存在问题，无法读取");
        }

        if (CollectionUtils.isEmpty(dataList)) {
            throw new BusinessException("数据为空");
        }

        return ApiResult.okMsg("成功导入" + dataList.size() + "条");
    }

    private void setOutage(PowerDebugEntity debug) {
        OutageWorkEntity entity = outageWorkDao.selectOne(Wrappers.<OutageWorkEntity>query().like("inspection_content", debug.getWorkLineName()).eq("date_format(start_date,'%Y-%m-%d')", DateUtil.format(debug.getWorkStartTime(), "yyyy-MM-dd")));
        if (entity != null) {
            outageWorkDao.update(Wrappers.<OutageWorkEntity>update().eq("id", entity.getId()).set("whether_debug", 1));
            debug.setOutageId(entity.getId());
        }
    }

    public PowerDebugVO getById(Integer id) {
        var form = new PowerDebugQueryForm();
        form.setPage(1L);
        form.setLimit(1L);
        form.setId(id);
        PageParam<PowerDebugEntity, PowerDebugQueryForm> page = new PageParam<>(form);
        return powerDebugDao.queryPage(page, form).stream().findFirst().orElse(null);
    }

    public List<Map<String,Object>> listByContentAndTime(List<OutageWorkEntity> substringMap) {
        return baseMapper.listByContentAndTime(substringMap);
    }
}
